<?php

namespace Drupal\migrate_retry;

use Drupal\migrate\MigrateMessageInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\MigrateExecutable as MigrateExecutableBase;
use Drupal\Component\EventDispatcher\Event;
use Drupal\migrate\Row;
use Drupal\migrate\MigrateSkipRowException;

/**
 * Defines a migrate executable class.
 */
class MigrateExecutable extends MigrateExecutableBase {

  /**
   * List of source IDs to import.
   *
   * @var array
   */
  protected array $idList;

  /**
   * Constructs a MigrateExecutable.
   *
   * @param MigrationInterface $migration
   *   The migration to run.
   * @param MigrateMessageInterface $message
   *   (optional) The migrate message service.
   * @param array $id_list
   *   (optional) List of source IDs to import.
   */
  public function __construct(MigrationInterface $migration, MigrateMessageInterface $message = NULL, array $id_list = []) {
    $this->idList = $id_list;
    $this->addListeners();
    parent::__construct($migration, $message);
  }

  /**
   * Add listeners related to this executable.
   */
  protected function addListeners(): void {
    $listeners['migrate_retry.prepare_row'] = [$this, 'onPrepareRow'];
    $dispatcher = $this->getEventDispatcher();
    foreach ($listeners as $event => $listener) {
      $dispatcher->addListener($event, $listener);
    }
  }

  /**
   * React to a new row.
   *
   * @param \Drupal\Component\EventDispatcher\Event $event
   *   The prepare-row event.
   *
   * @throws \Drupal\migrate\MigrateSkipRowException
   */
  public function onPrepareRow(Event $event): void {
    if (!empty($this->idList)) {
      $source_ids = $event->row instanceof Row ? $event->row->getSourceIdValues() : NULL;
      if (!in_array($source_ids, $this->idList, TRUE)) {
        throw new MigrateSkipRowException('Skipped, due to migrate retry id list.', FALSE);
      }
    }
  }

}
